@@ -1,73 +1,35 @@
 """
-Created on May 23, 2010
-
-@author: Nicklas Boerjesson
-@note: 
-* The mySQL DDL implementation defaults to using innoDB, since stuff like foreign keys and other very important security features are lacking from myISAM.
-* All Parameter_Base descendants property names are named in a specific way, so that one from that name can discern what types are allowed.
-For example: sources means that it is a list of Parameter_Source.
-* Parameter_ means that it is some form of input, Verb_ means that this statement can be executed stand alone.
- 
-@warning: Changes and new classes must satisfy both the import/export of data structures and schema generation.
+    ***********************************************************
+    qal.sql.sql holds the class structure representation of SQL
+    ***********************************************************
+    .. note:: 
+        * The mySQL DDL implementation defaults to using innoDB, since stuff like foreign keys and other very important security features are lacking from myISAM.
+        * All Parameter_Base descendants property names are named in a specific way, so that one from that name can discern what types are allowed. For example: sources means that it is a list of Parameter_Source.
+        * Parameter_* means that it is some form of input, Verb_* means that this statement can be executed stand alone.
 
+    :warning: Changes and new classes must satisfy both the import/export of data structures and schema generation.
+    :copyright: Copyright 2010-2013 by Nicklas Boerjesson
+    :license: BSD, see LICENSE for details.
 """
-from qal.sql.sql_types import DEFAULT_ROWSEP, expression_item_types,tabular_expression_item_types
-from qal.dal.dal_types import DB_POSTGRESQL, DB_MYSQL, DB_ORACLE, DB_DB2, DB_SQLSERVER 
 
-from qal.sql.sql_utils import *
-from qal.sql.sql_types import condition_part
-from qal.dataset.flatfile import Parameter_Flatfile_Dataset
-from qal.dataset.rdbms import Parameter_RDBMS_Dataset
-from qal.dataset.xml import Parameter_XML_Dataset
-from qal.dataset.matrix import Parameter_Matrix_Dataset
-                              
-class Parameter_Base(object): 
-    """This class is a base class for all parameter classes."""
-    _row_separator = DEFAULT_ROWSEP 
+from qal.sql.types import DEFAULT_ROWSEP, expression_item_types,tabular_expression_item_types
+from qal.dal.types import DB_POSTGRESQL, DB_MYSQL, DB_ORACLE, DB_DB2 # , DB_SQLSERVER 
 
-    def __init__(self, _row_separator = None):
-        super(Parameter_Base, self ).__init__()
-        if _row_separator != None: 
-            self._row_separator = _row_separator
+from qal.sql.base import Parameter_Base, SQL_List, Parameter_Expression_Item
+from qal.sql.remotable import Parameter_Remotable
 
-    def as_sql(self, _db_type): 
-        """Generate SQL for specified database engine"""
-        raise Exception(self.__class__.__name__ + ".as_sql() is not implemented")
- 
- 
-class SQL_List(list):
-    """This is the base class for lists of class instances."""    
 
-    def __init__(self, _itemclasses = None):
-        super(SQL_List, self ).__init__()
-        if _itemclasses != None:
-            self._itemclasses = _itemclasses
+from qal.sql.utils import add_operator, parenthesise, oracle_add_escape, add_comma, make_operator, check_for_param_content,\
+                                none_as_sql, error_on_blank, comma_separate, make_function, db_specific_operator, db_specific_object_reference,\
+                                citate,check_not_null, curr_user, db_specific_datatype, curr_datetime, add_comma_rs, oracle_create_auto_increment,\
+                                handle_temp_table_ref, datatype_to_parameter
+from qal.sql.types import condition_part
+from qal.dataset.flatfile import Flatfile_Dataset
+from qal.dataset.xpath import XPath_Dataset
+from qal.dataset.matrix import Matrix_Dataset
+from qal.dal.types import DB_SQLSERVER
 
-    def as_sql(self, _db_type): 
-        """Generate SQL for specified database engine"""
-        result = '' 
-        for _item in self:
-#            TODO: Check if the below list functionality is ever needed/wanted.             
-#            result+= DEFAULT_ROWSEP + '--****************' + str(self.itemclasses) + ' number ' + index +'****************************' + DEFAULT_ROWSEP 
-            if hasattr(_item, 'as_sql'):
-                result+= _item.as_sql(_db_type)
-            else:
-                result+= _item
 
-        return result
-
-   
-class Parameter_Expression_Item(Parameter_Base):
-    """The superclass of all classes that can be considered part of an expression"""
-    operator = 'C'  
-    
-    def __init__(self,_operator = None):
-        super(Parameter_Expression_Item, self ).__init__()
-        if _operator != None:
-            self.operator = _operator
-        else:
-            self.operator = 'C' 
-    
 
 class Parameter_Expression(Parameter_Expression_Item):
     """Holds an expression"""
@@ -82,7 +44,7 @@ class Parameter_Expression(Parameter_Expression_Item):
         else:
             self.expressionitems = SQL_List()
                    
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine""" 
         result = ''
         #Loop expressions
@@ -113,7 +75,7 @@ class Parameter_String(Parameter_Expression_Item):
             self.escape_character = ''
         
     
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         if _db_type != DB_ORACLE:
             return "'" + self.string_value + "'"
@@ -129,7 +91,7 @@ class Parameter_Numeric(Parameter_Expression_Item):
         super(Parameter_Numeric, self ).__init__(_operator)
         self.numeric_value = _numeric_value  
     
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         
         #Check if it is a valid float
@@ -140,6 +102,23 @@ class Parameter_Numeric(Parameter_Expression_Item):
                 raise Exception("A numeric parameter must be numeric. Value:" + self.numeric_value)
         return str(self.numeric_value)   
     # TODO: Check if any flavors allow other decimal signs.
+
+class Parameter_Parameter(Parameter_Expression_Item):
+    """Holds a parameter to be used in prepared statements."""
+    
+    datatype = ''  
+  
+    def __init__(self, _datatype = '', _operator = None):
+        super(Parameter_Parameter, self ).__init__(_operator)
+        self.datatype = _datatype  
+    
+
+    def _generate_sql(self, _db_type):
+        """Generate SQL for specified database engine"""
+       
+        # This is temporary, awaiting proper PyMySQL support for %d.
+        return datatype_to_parameter(_db_type, self.datatype)
+
     
 class Parameter_IN(Parameter_Expression_Item):
     """Hold an SQL IN-statement"""
@@ -149,7 +128,7 @@ class Parameter_IN(Parameter_Expression_Item):
         super(Parameter_IN, self ).__init__(_operator)
         self.in_values = _in_values 
     
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         
         # Handle strings separately. Somewhat non-pythonic, but the alternative would be far more complicated.
@@ -163,8 +142,8 @@ class Parameter_IN(Parameter_Expression_Item):
 
         
 
-class Parameter_Dataset(Parameter_Expression_Item):  
-    """Holds a dataset from an external source"""
+class Parameter_Dataset(Parameter_Expression_Item, Parameter_Remotable):  
+    """Holds a dataset from an external, non-SQL source"""
     data_source = None
 
     
@@ -175,6 +154,8 @@ class Parameter_Dataset(Parameter_Expression_Item):
             self.data_source = _data_source
         else:
             self.data_source = None
+        
+        self.resource_uuid = None
             
     def dataset_to_sql(self):
         """Generate SQL for specified database engine """
@@ -190,13 +171,13 @@ class Parameter_Dataset(Parameter_Expression_Item):
         else:
             raise Exception('Dataset_To_SQL: Dataset not loaded.'); 
         
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         if (self.data_source):
            
             self.data_source.load()
             return "("+ self.data_source.as_sql(_db_type) + ")"       
         else:
-            raise Exception('Parameter_Dataset.as_sql : data_source not set.');
+            raise Exception('Parameter_Datasetas_sql : data_source not set.');
 
 class Parameter_Identifier(Parameter_Expression_Item):
     """Holds an identifier(column-, table or other reference)"""
@@ -211,8 +192,9 @@ class Parameter_Identifier(Parameter_Expression_Item):
         else:
             self.prefix = '' 
             
+                                  
     
-    def as_sql(self, _db_type, _is_source_table = False):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         # Add prefix if set
         if self.prefix != '':
@@ -220,10 +202,10 @@ class Parameter_Identifier(Parameter_Expression_Item):
         else:
             _tmp_prefix = ''
         # Add quote if needed
-        if _db_type in (DB_POSTGRESQL, DB_DB2, DB_ORACLE) and not _is_source_table:
-            return  _tmp_prefix + '"' + str(self.identifier) + '"'
+        if _db_type in (DB_POSTGRESQL, DB_DB2, DB_ORACLE):
+            return _tmp_prefix + '"' + str(handle_temp_table_ref(self.identifier, _db_type)) + '"'
         else:    
-            return _tmp_prefix + str(self.identifier)
+            return _tmp_prefix + str(handle_temp_table_ref(self.identifier, _db_type))
 
 
 class Parameter_Cast(Parameter_Expression_Item):
@@ -251,7 +233,7 @@ class Parameter_Cast(Parameter_Expression_Item):
         
         self.datatype = _datatype
     
-    def as_sql(self, _DB_Type):
+    def _generate_sql(self, _DB_Type):
         """Generate SQL for specified database engine"""
         return self.make_cast(self.expression.as_sql(_DB_Type))    
 
@@ -271,7 +253,7 @@ class Parameter_Function(Parameter_Expression_Item):
         self.name = _name    
 
 
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         result = ''
         for index, item in enumerate(self.parameters):
@@ -289,7 +271,7 @@ class Parameter_WHEN(Parameter_Base):
         self.conditions = _conditions
         self.result = _result  
         
-    def as_sql(self,_db_type):
+    def _generate_sql(self,_db_type):
         """Generate SQL for specified database engine"""
         return 'WHEN ' + self.conditions.as_sql(_db_type) + ' THEN ' + self.result.as_sql(_db_type)
 
@@ -309,7 +291,7 @@ class Parameter_CASE(Parameter_Expression_Item):
         
         self.else_statement = _else_statement
 
-    def as_sql(self, _DB_Type):
+    def _generate_sql(self, _DB_Type):
         """Generate SQL for specified database engine"""
         result = 'CASE'
         for item in self.when_statements:
@@ -324,7 +306,7 @@ class Parameter_CASE(Parameter_Expression_Item):
 class Parameter_Set(Parameter_Base): 
     
     """This class holds a set. 
-    In SQL, that means more than one tabular datasets combiner using a set operator like UNION."""
+    In SQL, that means more than one tabular datasets is combined using a set operator like UNION."""
     # expression could be any table-valued expression
     subsets     = None
     set_operator = None
@@ -342,7 +324,7 @@ class Parameter_Set(Parameter_Base):
             self.set_operator = _set_operator
         else:
             self.set_operator = None                
-    def as_sql(self,_db_type):
+    def _generate_sql(self,_db_type):
         """Generate SQL for specified database engine"""
         _sqls = []
         # Loop all expressions into a list. 
@@ -350,8 +332,8 @@ class Parameter_Set(Parameter_Base):
         # Separate them with operators.
         return ('\n'+self.set_operator + '\n').join(_sqls)     
     
-class Parameter_Source(Parameter_Base):
-    """This class defines any source of data that can be used with a FROM or JOIN-statement."""
+class Parameter_Source(Parameter_Base, Parameter_Remotable):
+    """This class hold a source of data that can be used with a FROM or JOIN-statement."""
     # expression could be any table-valued expression
     expression     = None
     
@@ -361,7 +343,7 @@ class Parameter_Source(Parameter_Base):
     def __init__(self, _expression = None, _conditions = None, _alias = '', _join_type = None):
         super(Parameter_Source, self ).__init__()
         
-        # One can either have tabular source data, or an SQL expression 
+        # One can either have tabular source data i union:ed sets, a sub select, an SQL identifier or a noSQL data set. 
 
         if _expression != None:
             self.expression = _expression
@@ -376,10 +358,13 @@ class Parameter_Source(Parameter_Base):
         self.alias = _alias
         
         self.join_type = _join_type
+        self.resource_uuid = None
         
-    def as_sql(self,_db_type):
+    def _generate_sql(self,_db_type):
         """Generate SQL for specified database engine"""
-        return none_as_sql(self.expression,_db_type, '')
+        _sql = none_as_sql(self.expression,_db_type, '')
+    
+        return _sql
     
 class Parameter_ORDER_BY_item(Parameter_Expression): 
     """This class holds an order by-statement"""
@@ -390,12 +375,12 @@ class Parameter_ORDER_BY_item(Parameter_Expression):
         super(Parameter_ORDER_BY_item, self ).__init__(_expressionitems)
         self.direction = _direction
 
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
-        return super(Parameter_ORDER_BY_item, self ).as_sql(_db_type) + " " + self.direction
+        return super(Parameter_ORDER_BY_item, self )._generate_sql(_db_type) + " " + self.direction
         
      
-class Verb_SELECT(Parameter_Expression_Item):
+class Verb_SELECT(Parameter_Expression_Item, Parameter_Remotable):
     """This class holds a SELECT statement. """
     fields = None
     sources = None
@@ -429,6 +414,7 @@ class Verb_SELECT(Parameter_Expression_Item):
             self.order_by = SQL_List("Parameter_Expression_Item")
             
         self.top_limit = None
+        self.resource_uuid = None
         
     def add_limit(self, _db_type):
         """Generate SQL for specified database engine for limits on number of rows (TOP/LIMIT/FETCH FIRST)"""
@@ -437,12 +423,12 @@ class Verb_SELECT(Parameter_Expression_Item):
                     self._post_sql = 'LIMIT ' + str(int(self.top_limit))
             elif (_db_type in [DB_DB2]):
                     self._post_sql = 'FETCH FIRST ' + str(int(self.top_limit)) + ' ROWS ONLY '         
-            # Oracles' solution is not handled here, but in the .as_sql-WHERE handling.
+            # Oracles' solution is not handled here, but in the as_sql-WHERE handling.
             elif (_db_type != DB_ORACLE): 
                 self._post_verb = 'TOP ' + str(int(self.top_limit)) + ' '
             
 
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         
         if len(self.sources) > 0:        
@@ -450,14 +436,18 @@ class Verb_SELECT(Parameter_Expression_Item):
             
         # Add select and its fields
         result = 'SELECT ' + self._post_verb
-        for index, item in enumerate(self.fields):
-            result+= add_comma(index, item.as_sql(_db_type))
+        if (self.fields != None) and (len(self.fields) > 0):
+            for index, item in enumerate(self.fields):
+                result+= add_comma(index, item.as_sql(_db_type))
+        else:
+            result+='*'
+            
         
         # Add FROM
         if len(self.sources) > 0:
             result+= ' FROM '
             result+= self.sources[0].as_sql(_db_type) 
-            if self.sources[0].alias != '':
+            if  self.sources[0].alias != None and self.sources[0].alias != '':
                 if (_db_type != DB_ORACLE):
                     result+= ' AS ' + self.sources[0].alias
                 else:
@@ -473,12 +463,16 @@ class Verb_SELECT(Parameter_Expression_Item):
         # Add joins
         if len(self.sources) > 1:
             for index, item in enumerate(self.sources):
+
                 if index > 0:
+                    """As as_sql is not called, as_sql has to be called explicitly."""
+                    item.as_sql(_db_type)
                     if item.join_type:
                         result+= ' '+ item.join_type
                     result+= ' JOIN ' + none_as_sql(item.expression, _db_type, _error = 'Verb_SELECT: Joins must contain a statement or a reference to a table.')
                     result+= ' AS ' + error_on_blank(item.alias,'Verb_SELECT: Joins must have aliases.')
-                    result+= ' ON ' + none_as_sql(item.conditions,_db_type, _error = 'Verb_SELECT: Joins must have conditions.')
+                    if item.join_type != "CROSS":
+                        result+= ' ON ' + none_as_sql(item.conditions,_db_type, _error = 'Verb_SELECT: Joins must have conditions.')
         
         # Add WHERE 
         if len(self.sources) > 0:
@@ -515,7 +509,7 @@ class Verb_SELECT(Parameter_Expression_Item):
         
         self._post_sql = ''
         self._post_verb = ''
-         
+
         return result
     
     def append_field_identifier(self, _identifier):
@@ -548,15 +542,43 @@ class Parameter_Condition(Parameter_Base):
         
     def as_sql(self, _db_type, _index = 0):
         """Generate SQL for specified database engine (index if for handling when it is the first in a list of conditions)"""
+        """Note that the Parameter_Base.as_sql is not called, this is because 1) Conditions are in context 2) Needs _index argument"""
         # TODO: Handle ILIKE for PostgreSQL.
         if (_index != 0):
+            error_on_blank(self.and_or ,'Parameter_Condition: Conditions must be separated by a logical operator. "and_or" is not set.')
             _result = ' ' + self.and_or + ' '
+                
         else:
             _result = ''
         
         _result+= parenthesise(self.left.as_sql(_db_type)  + ' ' + db_specific_operator(self.operator, _db_type) + ' ' + self.right.as_sql(_db_type))  
         return _result
 
+class Parameter_Assignment(Parameter_Base):
+    """This class holds a assignment, that is assign the result of the expression to the right to the identifier on the left."""
+    left = None
+    right = None
+
+    
+    def __init__(self, _left = None, _right = None):
+        super(Parameter_Assignment, self ).__init__()
+        if _left != None:
+            self.left     = _left
+        else:
+            self.left     = None
+        if _right != None:
+            self.right     = _right
+        else:
+            self.right     = None
+
+    def as_sql(self, _db_type):
+        """Generate SQL for specified database engine"""
+        # TODO: Handle ILIKE for PostgreSQL.
+        
+        _result = self.left.as_sql(_db_type)  + ' = ' + self.right.as_sql(_db_type)  
+        return _result
+        
+
 class Parameter_Conditions(SQL_List):
     """This class holds a list of condition or list of conditions ((A=B AND C=D) OR (E=F))"""
     def __init__(self):
@@ -564,7 +586,7 @@ class Parameter_Conditions(SQL_List):
    
    
     def get_first_and_or(self):
-        """Return the forst and/or to know if it should add its own
+        """Return the first and/or to know if it should add its own
         (If it is the first condition there is no point in adding its or, but rather the parent conditions' operator."""
         if (len(self)) > 0:
             _first_item = self[0]
@@ -592,7 +614,7 @@ class Parameter_Conditions(SQL_List):
         return _result
 
 class Parameter_Field(Parameter_Base):
-    """Holds a field definition (SELECT _FIELD1 AS FIELD_) """
+    """Holds a field definition (SELECT _FIELD1 AS _FIELD) """
     expression = None
     alias = ''
     
@@ -604,7 +626,7 @@ class Parameter_Field(Parameter_Base):
             self.expression = _expression   
         self.alias      = _alias
     
-    def as_sql(self, _DB_Type):
+    def _generate_sql(self, _DB_Type):
         """Generate SQL for specified database engine"""
         if self.alias != '':
             return self.expression.as_sql(_DB_Type) + ' AS ' + self.alias
@@ -633,7 +655,7 @@ class Parameter_Constraint(Parameter_Base):
             self.checkconditions = SQL_List(["Parameter_Condition", "Parameter_Conditions"])
 
                 
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         result = 'CONSTRAINT ' + db_specific_object_reference(self.name, _db_type) + ' ' + self.constraint_type
         if (self.constraint_type == "CHECK"):
@@ -668,7 +690,7 @@ class Verb_CREATE_INDEX(Parameter_Base):
         else:
             self.columnnames = SQL_List('string')
           
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         check_not_null("Verb_CREATE_INDEX", [[self.name, "name"],[self.index_type, "index_type"], [self.tablename, "tablename"]])
         # Handle DB2s strange deviation #1
@@ -705,7 +727,7 @@ class Parameter_ColumnDefinition(Parameter_Base):
   
         
 
-    def as_sql(self, _db_type, _mysql_pk = False):
+    def _generate_sql(self, _db_type, _mysql_pk = False):
         """Generate SQL for specified database engine"""
         result = db_specific_object_reference(self.name, _db_type) + ' ' + db_specific_datatype(self.datatype, _db_type) 
         if (_db_type == DB_MYSQL and _mysql_pk == True and self.datatype.lower() == 'serial'):
@@ -767,6 +789,7 @@ class Verb_CREATE_TABLE(Parameter_DDL):
         
         self._post_statements = list()
         
+        
     def get_post_statements(self):
         """Return post statements. That is statements run after the main SQL"""
         return self._post_statements    
@@ -795,20 +818,34 @@ class Verb_CREATE_TABLE(Parameter_DDL):
 
 
     
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         self._post_statements = []
-        result = 'CREATE TABLE '+ citate(self.name, _db_type) + ' (' + self._row_separator
-        result+= self.make_columns(_db_type) 
+        if self.name[0] == "#":
+            if _db_type in [DB_POSTGRESQL, DB_MYSQL]:
+                _result = "CREATE TEMPORARY TABLE "
+            elif _db_type == [DB_DB2]:
+                _result = "DECLARE GLOBAL TEMPORARY TABLE "
+            elif _db_type == [DB_ORACLE]:
+                _result = "CREATE GLOBAL TEMPORARY TABLE "
+            else:
+                _result = "CREATE TABLE "
+            
+            _result+= citate(self.name[1:len(self.name)], _db_type) + " (" + self._row_separator
+        else:
+            _result = "CREATE TABLE "
+            _result+= citate(self.name, _db_type) + " (" + self._row_separator
+            
+        _result+= self.make_columns(_db_type) 
         if (len(self.constraints) > 0):
-            result+= ',' + self._row_separator + self.make_constraints(_db_type) 
-        result+= self._row_separator + ')' 
+            _result+= ',' + self._row_separator + self.make_constraints(_db_type) 
+        _result+= self._row_separator + ')' 
         
   
         if _db_type == DB_MYSQL:
-            result+= ' ENGINE=InnoDB'
+            _result+= ' ENGINE=InnoDB'
                  
-        return result
+        return _result
     
 class Verb_INSERT(Parameter_Base):
     """This class holds an INSERT statement"""
@@ -843,33 +880,98 @@ class Verb_INSERT(Parameter_Base):
             result = result + currIdent.as_sql(_db_type)
         return result     
         
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         
         if len(self.column_identifiers) > 0:
             result = 'INSERT INTO '+ self.destination_identifier.as_sql(_db_type) + ' (' + self.makeIdentifiers(_db_type) + ')' + DEFAULT_ROWSEP
             # Add data and remove parenthesis. Removing them here could certainly be seen as ugly but the reason 
             # is that MySQL cannot handle them. It would break everything else any other way.
-            tmpSQL = self.data.as_sql(_db_type)
-            if isinstance(self.data, Parameter_Set):
-                result = result +  tmpSQL
-            else:
-                result = result +  tmpSQL.lstrip('(').rstrip(')')
+            if self.data:
+                tmpSQL = self.data.as_sql(_db_type)
+                if isinstance(self.data, Parameter_Set):
+                    result = result +  tmpSQL
+                else:
+                    result = result +  tmpSQL.lstrip('(').rstrip(')')
         else:
             raise Exception('Verb_INSERT.as_sql: No column_identifiers specified!')
                    
         return result  
     
-class Verb_DELETE(Parameter_Expression_Item):
+class Verb_UPDATE(Parameter_Base):
+    """This class holds an INSERT statement"""
+    
+    table_identifier = None
+    assignments = None
+    conditions = None
+
+    def __init__(self, _table_identifier = None, _assignments = None, _conditions = None):
+        super(Verb_UPDATE, self ).__init__()
+        if _table_identifier != None:
+            self.table_identifier = _table_identifier  
+        else:
+            self.table_identifier = None
+        
+        if _assignments != None:
+            self.assignments    = _assignments
+        else:
+            self.assignments    = SQL_List("Parameter_Assignment")
+                               
+        if _conditions != None:        
+            self.conditions = _conditions
+        else:
+            self.conditions = Parameter_Conditions()            
+
+    def _generate_assignments_sql(self, _db_type):
+        _results = [] 
+        for _curr_assignment in self.assignments:
+            _results.append( _curr_assignment.as_sql(_db_type))
+        
+        return ", ".join(_results)
+        
+    def _generate_sql(self, _db_type):
+        """Generate SQL for specified database engine"""
+        
+        if len(self.assignments) > 0:
+            _result = 'UPDATE '+ self.table_identifier.as_sql(_db_type) + DEFAULT_ROWSEP
+            _result+= 'SET' + DEFAULT_ROWSEP + self._generate_assignments_sql(_db_type)+ DEFAULT_ROWSEP 
+            _result+= 'WHERE ' + self.conditions.as_sql(_db_type)
+        else:
+            raise Exception('.as_sql(_db_type) + DEFAULT_ROWSEP.as_sql: No column_identifiers specified!')
+                   
+        return _result  
+    
+class Verb_DROP_TABLE(Parameter_Base):
+    """This class holds a DELETE statement"""
+    name = None
+    """
+       The name of the table to be dropped.
+    """
+
+    
+    def __init__(self, _name): 
+        super(Verb_DROP_TABLE, self ).__init__()
+            
+        if _name != None:
+            self.name = _name
+        else:
+            self.name = None
+        
+
+    def _generate_sql(self, _db_type):
+        """Generate SQL for specified database engine"""
+        return 'DROP TABLE ' + citate(self.name, _db_type)
+
+class Verb_DELETE(Parameter_Base):
     """This class holds a DELETE statement"""
     sources = None
     """
-       @warning: Important! The first source must have a parameter_identifier that specifies the target table.
+       @warning: Important! The first source must have a parameter_identifier as expression that specifies the target table.
     """
 
     
     def __init__(self, _sources = None, _operator = None): 
-        super(Verb_DELETE, self ).__init__(_operator)
+        super(Verb_DELETE, self ).__init__()
         
             
         if _sources != None:
@@ -878,7 +980,7 @@ class Verb_DELETE(Parameter_Expression_Item):
             self.sources = SQL_List("Parameter_Source")
         
 
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Generate SQL for specified database engine"""
         
         result = ''
@@ -952,7 +1054,7 @@ class Verb_Custom(Parameter_DDL):
     sql_db2 = ''
     sql_sqlserver = ''
 
-    def as_sql(self, _db_type):
+    def _generate_sql(self, _db_type):
         """Return the specified SQLs for each database engine"""
         return [self.sql_mysql, self.sql_postgresql, self.sql_oracle, self.sql_db2, self.sql_sqlserver][_db_type]
         
