@@ -4,11 +4,11 @@ Created on May 23, 2010
 @author: Nicklas Boerjesson
 '''
 import unittest
-import difflib
-from qal.sql.sql_types import *
-from qal.dal.dal_types import *
-from qal.sql.sql import *
 
+from qal.sql.types import *
+from qal.dal.types import *
+from qal.sql.sql import *
+#from qal.tools.diff import diff_strings  # Use when debugging tests
 
 global r_create_table_mysql 
 r_create_table_mysql = "CREATE TABLE Table1 ("+DEFAULT_ROWSEP + "\
@@ -84,20 +84,17 @@ r_SELECT_oracle = "SELECT (T1.\"CountryPrefix\" + '+' + T1.\"PhoneNumber\" + Sim
 global r_SELECT_SQL_Server 
 r_SELECT_SQL_Server= "SELECT TOP 1 (T1.CountryPrefix + '+' + T1.PhoneNumber + Simple(CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix + '+' + T2.PhoneNumber))) AS Field1, (T2.CountryPrefix + '+' + T2.PhoneNumber) AS Field2 FROM testtable AS T1 JOIN testtable AS T2 ON ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')) WHERE ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')) ORDER BY T1.Field1 desc, T2.Field1 asc"
 
+global r_UPDATE_my_sql
+r_UPDATE_my_sql = "SET"+DEFAULT_ROWSEP+ "dest_column = 'Hello'" + DEFAULT_ROWSEP + "WHERE ((col_1 = '1') AND (col_2 = '1'))"
+global r_UPDATE_DB2
+r_UPDATE_DB2 = "SET" + DEFAULT_ROWSEP + "\"dest_column\" = 'Hello'" + DEFAULT_ROWSEP + "WHERE ((\"col_1\" = '1') AND (\"col_2\" = '1'))"
+global r_UPDATE_postgresql
+r_UPDATE_postgresql = "SET" + DEFAULT_ROWSEP + "\"dest_column\" = 'Hello'" + DEFAULT_ROWSEP + "WHERE ((\"col_1\" = '1') AND (\"col_2\" = '1'))"
+global r_UPDATE_oracle
+r_UPDATE_oracle = "SET" + DEFAULT_ROWSEP + "\"dest_column\" = 'Hello'" + DEFAULT_ROWSEP + "WHERE ((\"col_1\" = '1') AND (\"col_2\" = '1'))"
+global r_UPDATE_SQL_Server
+r_UPDATE_SQL_Server = "SET" + DEFAULT_ROWSEP + "dest_column = 'Hello'" + DEFAULT_ROWSEP + "WHERE ((col_1 = '1') AND (col_2 = '1'))"
 
-# Utils
-
-def my_diff(a,b):
-    result= '---------- String A-----------' + DEFAULT_ROWSEP
-    result+= a + DEFAULT_ROWSEP
-    result+= '---------- String B-----------' + DEFAULT_ROWSEP
-    result+= b + DEFAULT_ROWSEP
- 
-    result+= '---------- Diff between A and B-----------' + DEFAULT_ROWSEP
-    for line in difflib.context_diff(a,b):
-        result+= line 
-        
-    return result
     
 
 # Generate test objects.
@@ -237,6 +234,23 @@ def gen_simple_insert():
     insert.column_identifiers.append(Parameter_Identifier("Table1Changed"))
     return insert
 
+def gen_simple_update():
+ 
+    _table_identifier = Parameter_Identifier("test")
+    
+    _a_1 = Parameter_Condition(Parameter_Identifier("col_1"), Parameter_String("1"), "=")
+    _a_2 = Parameter_Condition(Parameter_Identifier("col_2"), Parameter_String("1"), "=", "AND")
+    _conditions = Parameter_Conditions()
+    _conditions.append(_a_1)
+    _conditions.append(_a_2)
+    
+    
+    _assignments = SQL_List()
+    _assignments.append(Parameter_Assignment(_left = Parameter_Identifier("dest_column"), _right = Parameter_String("Hello")))
+
+    _update = Verb_UPDATE(_table_identifier =Parameter_Identifier("test"),_assignments = _assignments, _conditions = _conditions)
+    return _update
+
 def gen_simple_create(): 
   
     col1_constraint1 = Parameter_Constraint('PK_Table1_Table1ID', "PRIMARY KEY", [Parameter_Identifier('Table1ID')])
@@ -268,96 +282,96 @@ def gen_simple_create():
 class parameter_test(unittest.TestCase):
 
 
-    def test_parameter_condition_simple(self):
+    def test_00_parameter_condition_simple(self):
         self.maxDiff = None
         param = gen_simple_condition_1()
         paramclass = param.__class__.__name__
-        testvalue = '(1.3 > 2.4)'
-        self.assertEqual(param.as_sql(DB_MYSQL), testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), testvalue, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        _testvalue = '(1.3 > 2.4)'
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), _testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _testvalue, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
 
-    def test_parameter_condition_complex(self):
+    def test_01_parameter_condition_complex(self):
         self.maxDiff = None
         param = gen_complex_conditions()
         paramclass = param.__class__.__name__
-        testvalue = "((1.3 > 2.4) AND (T1.firstname LIKE '%icklas') AND ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')))"
-        testvalue_Oracle = "((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas') AND ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')))"
-        testvalue_PostgreSQL = "((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas') AND ((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas')))"
-        testvalue_DB2 = "((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas') AND ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')))"
+        _testvalue = "((1.3 > 2.4) AND (T1.firstname LIKE '%icklas') AND ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')))"
+        _testvalue_Oracle = "((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas') AND ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')))"
+        _testvalue_PostgreSQL = "((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas') AND ((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas')))"
+        _testvalue_DB2 = "((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas') AND ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')))"
 
-        self.assertEqual(param.as_sql(DB_MYSQL), testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), testvalue_Oracle, paramclass +'.as_sql(DB_ORACLE) failed.')
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), testvalue_PostgreSQL, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), testvalue_DB2, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _testvalue_Oracle, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), _testvalue_PostgreSQL, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _testvalue_DB2, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
 
-    def test_parameter_expression_simple(self):
+    def test_02_parameter_expression_simple(self):
         self.maxDiff = None
         param = gen_simpleexpression_1()
-        testvalue = "(T1.CountryPrefix + '+' + T1.PhoneNumber + Simple(CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix + '+' + T2.PhoneNumber)))"
-        pipetestvalue = "(T1.CountryPrefix || '+' || T1.PhoneNumber || Simple(CAST((T2.CountryPrefix || '+' || T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix || '+' || T2.PhoneNumber)))"
-        postgrestestvalue = "(T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\")))"
-        db2testvalue = "(T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\")))"
-        oracletestvalue = "(T1.\"CountryPrefix\" + '+' + T1.\"PhoneNumber\" + Simple(CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\")))"
+        _testvalue = "(T1.CountryPrefix + '+' + T1.PhoneNumber + Simple(CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix + '+' + T2.PhoneNumber)))"
+        pipe_testvalue = "(T1.CountryPrefix || '+' || T1.PhoneNumber || Simple(CAST((T2.CountryPrefix || '+' || T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix || '+' || T2.PhoneNumber)))"
+        _postgres_testvalue = "(T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\")))"
+        _db2_testvalue = "(T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\")))"
+        _oracle_testvalue = "(T1.\"CountryPrefix\" + '+' + T1.\"PhoneNumber\" + Simple(CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\")))"
         paramclass = param.__class__.__name__
 
-        self.assertEqual(param.as_sql(DB_MYSQL), testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), oracletestvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), postgrestestvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), db2testvalue, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _oracle_testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), _postgres_testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _db2_testvalue, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
 
-    def test_parameter_cast_simple(self):
+    def test_03_parameter_cast_simple(self):
         self.maxDiff = None
         param = gen_simple_cast()
-        testvalue = "CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200))"
-        db2testvalue = "CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200))"
-        postgrestestvalue = "CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200))"
-        oracletestvalue = "CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200))"
+        _testvalue = "CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200))"
+        _db2_testvalue = "CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200))"
+        _postgres_testvalue = "CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200))"
+        _oracle_testvalue = "CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200))"
         paramclass = param.__class__.__name__
         
-        self.assertEqual(param.as_sql(DB_MYSQL), testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), oracletestvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), postgrestestvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), db2testvalue, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _oracle_testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), _postgres_testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _db2_testvalue, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
         
-    def test_parameter_function_simple(self):
+    def test_04_parameter_function_simple(self):
         self.maxDiff = None
         param = gen_simple_function()
-        testvalue = "Simple(CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix + '+' + T2.PhoneNumber))"
-        db2testvalue = "Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))"
-        postgrestestvalue = "Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))"
-        oracletestvalue = "Simple(CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\"))"
+        _testvalue = "Simple(CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix + '+' + T2.PhoneNumber))"
+        _db2_testvalue = "Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))"
+        _postgres_testvalue = "Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))"
+        _oracle_testvalue = "Simple(CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\"))"
         paramclass = param.__class__.__name__
 
-        self.assertEqual(param.as_sql(DB_MYSQL), testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), oracletestvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), postgrestestvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), db2testvalue, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _oracle_testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), _postgres_testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _db2_testvalue, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
 
 
-    def test_parameter_case(self):
+    def test_05_parameter_case(self):
         self.maxDiff = None
         param = gen_simple_case()
-        testvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')) THEN (T1.CountryPrefix + '+' + T1.PhoneNumber + Simple(CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix + '+' + T2.PhoneNumber))) WHEN ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')) THEN (T2.CountryPrefix + '+' + T2.PhoneNumber) else_statement (T2.CountryPrefix + '+' + T2.PhoneNumber) END"
-        PostgreSQLtestvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas')) THEN (T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))) WHEN ((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas')) THEN (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") else_statement (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") END"
-        DB2testvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))) WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") else_statement (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") END"
-        Oracletestvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T1.\"CountryPrefix\" + '+' + T1.\"PhoneNumber\" + Simple(CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\"))) WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") else_statement (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") END"
+        _testvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')) THEN (T1.CountryPrefix + '+' + T1.PhoneNumber + Simple(CAST((T2.CountryPrefix + '+' + T2.PhoneNumber) AS varchar(200)), (T2.CountryPrefix + '+' + T2.PhoneNumber))) WHEN ((1.3 > 2.4) AND (T1.firstname LIKE '%icklas')) THEN (T2.CountryPrefix + '+' + T2.PhoneNumber) else_statement (T2.CountryPrefix + '+' + T2.PhoneNumber) END"
+        PostgreSQL_testvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas')) THEN (T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))) WHEN ((1.3 > 2.4) AND (T1.\"firstname\" ILIKE '%icklas')) THEN (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") else_statement (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") END"
+        _db2_testvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T1.\"CountryPrefix\" || '+' || T1.\"PhoneNumber\" || Simple(CAST((T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\"))) WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") else_statement (T2.\"CountryPrefix\" || '+' || T2.\"PhoneNumber\") END"
+        _oracle_testvalue = "CASE WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T1.\"CountryPrefix\" + '+' + T1.\"PhoneNumber\" + Simple(CAST((T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") AS varchar(200)), (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\"))) WHEN ((1.3 > 2.4) AND (T1.\"firstname\" LIKE '%icklas')) THEN (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") else_statement (T2.\"CountryPrefix\" + '+' + T2.\"PhoneNumber\") END"
         paramclass = param.__class__.__name__
 
-        self.assertEqual(param.as_sql(DB_MYSQL), testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), Oracletestvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _oracle_testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
       
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), PostgreSQLtestvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), DB2testvalue, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), PostgreSQL_testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _db2_testvalue, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
 
-    def test_parameter_SELECT(self):
+    def test_06_VERB_SELECT(self):
         self.maxDiff = None
         param = gen_simple_select()
         paramclass = param.__class__.__name__
@@ -368,7 +382,7 @@ class parameter_test(unittest.TestCase):
         self.assertEqual(param.as_sql(DB_DB2), r_SELECT_DB_DB2, paramclass +'.as_sql(DB_DB2) failed.')
         self.assertEqual(param.as_sql(DB_SQLSERVER), r_SELECT_SQL_Server, paramclass +'.as_sql(DB_SQLSERVER) failed.')
 
-    def test_parameter_create_table(self):
+    def test_07_parameter_create_table(self):
         self.maxDiff = None
         param = gen_simple_create()
         paramclass = param.__class__.__name__
@@ -378,36 +392,68 @@ class parameter_test(unittest.TestCase):
         self.assertEqual(param.as_sql(DB_DB2), r_create_table_db2, paramclass +'.as_sql(DB_DB2) failed.')
         self.assertEqual(param.as_sql(DB_SQLSERVER), r_create_table_sqlserver, paramclass +'.as_sql(DB_SQLSERVER) failed.')
 
-    def test_parameter_insert(self):
+    def test_08_VERB_INSERT(self):
         self.maxDiff = None
         param = gen_simple_insert()
 
-        testvalue = "INSERT INTO test (Table1ID, Table1Name, Table1Changed)"
-        quotedtestvalue = "INSERT INTO \"test\" (\"Table1ID\", \"Table1Name\", \"Table1Changed\")"
-        paramclass = param.__class__.__name__#        self.assertEqual(param.as_sql(DB_MYSQL), testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        _testvalue = "INSERT INTO test (Table1ID, Table1Name, Table1Changed)"
+        quoted_testvalue = "INSERT INTO \"test\" (\"Table1ID\", \"Table1Name\", \"Table1Changed\")"
+        paramclass = param.__class__.__name__#        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
 
-        self.assertEqual(param.as_sql(DB_MYSQL), testvalue + DEFAULT_ROWSEP + r_SELECT_SQL, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), quotedtestvalue + DEFAULT_ROWSEP + r_SELECT_oracle, paramclass +'.as_sql(DB_ORACLE) failed.')
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), quotedtestvalue + DEFAULT_ROWSEP + r_SELECT_postgresql, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), quotedtestvalue + DEFAULT_ROWSEP + r_SELECT_DB_DB2, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), testvalue + DEFAULT_ROWSEP + r_SELECT_SQL_Server, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue + DEFAULT_ROWSEP + r_SELECT_SQL, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), quoted_testvalue + DEFAULT_ROWSEP + r_SELECT_oracle, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), quoted_testvalue + DEFAULT_ROWSEP + r_SELECT_postgresql, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), quoted_testvalue + DEFAULT_ROWSEP + r_SELECT_DB_DB2, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue + DEFAULT_ROWSEP + r_SELECT_SQL_Server, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        
+    def test_09_VERB_UPDATE(self):
+        self.maxDiff = None
+        param = gen_simple_update()
+
+        _testvalue = "UPDATE test"
+        quoted_testvalue = 'UPDATE "test"'
+        paramclass = param.__class__.__name__#        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+
+        self.assertEqual(param.as_sql(DB_MYSQL), _testvalue + DEFAULT_ROWSEP + r_UPDATE_my_sql, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), quoted_testvalue + DEFAULT_ROWSEP + r_UPDATE_oracle, paramclass +'.as_sql(DB_ORACLE) failed.')
+        
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), quoted_testvalue + DEFAULT_ROWSEP + r_UPDATE_postgresql, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), quoted_testvalue + DEFAULT_ROWSEP + r_UPDATE_DB2, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _testvalue + DEFAULT_ROWSEP + r_UPDATE_SQL_Server, paramclass +'.as_sql(DB_SQLSERVER) failed.')
         
 
-    def test_parameter_create_index(self):
+
+    def test_10_parameter_create_index(self):
         self.maxDiff = None
         param = Verb_CREATE_INDEX('ind_Table1ID', "CLUSTERED", 'Table1', ['Table1Name', 'Table1Date'])
         paramclass = param.__class__.__name__
-        mysqltestvalue      = 'CREATE CLUSTERED INDEX `ind_Table1ID`' + param._row_separator + 'ON Table1(`Table1Name`, `Table1Date`)'
-        oracletestvalue     = 'CREATE CLUSTERED INDEX "ind_Table1ID"' + param._row_separator + 'ON Table1("Table1Name", "Table1Date")'
-        postgrestestvalue   = 'CREATE CLUSTERED INDEX "ind_Table1ID"' + param._row_separator + 'ON Table1("Table1Name", "Table1Date")'
-        db2testvalue        = 'CREATE INDEX "ind_Table1ID"' + param._row_separator + 'ON Table1("Table1Name", "Table1Date")' + param._row_separator + 'CLUSTER'
-        sqlservertestvalue  = 'CREATE CLUSTERED INDEX [ind_Table1ID]' + param._row_separator + 'ON Table1([Table1Name], [Table1Date])'
-        self.assertEqual(param.as_sql(DB_MYSQL), mysqltestvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
-        self.assertEqual(param.as_sql(DB_ORACLE), oracletestvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
-        self.assertEqual(param.as_sql(DB_POSTGRESQL), postgrestestvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
-        self.assertEqual(param.as_sql(DB_DB2), db2testvalue, paramclass +'.as_sql(DB_DB2) failed.')
-        self.assertEqual(param.as_sql(DB_SQLSERVER), sqlservertestvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+        _mysql_testvalue      = 'CREATE CLUSTERED INDEX `ind_Table1ID`' + param._row_separator + 'ON Table1(`Table1Name`, `Table1Date`)'
+        _oracle_testvalue     = 'CREATE CLUSTERED INDEX "ind_Table1ID"' + param._row_separator + 'ON Table1("Table1Name", "Table1Date")'
+        _postgres_testvalue   = 'CREATE CLUSTERED INDEX "ind_Table1ID"' + param._row_separator + 'ON Table1("Table1Name", "Table1Date")'
+        _db2_testvalue        = 'CREATE INDEX "ind_Table1ID"' + param._row_separator + 'ON Table1("Table1Name", "Table1Date")' + param._row_separator + 'CLUSTER'
+        _sqlserver_testvalue  = 'CREATE CLUSTERED INDEX [ind_Table1ID]' + param._row_separator + 'ON Table1([Table1Name], [Table1Date])'
+        self.assertEqual(param.as_sql(DB_MYSQL), _mysql_testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _oracle_testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), _postgres_testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _db2_testvalue, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _sqlserver_testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
        
+    def test_11_parameter_drop_table(self):
+        self.maxDiff = None
+        param = Verb_DROP_TABLE('test')
+        paramclass = param.__class__.__name__
+        _mysql_testvalue      = 'DROP TABLE test'
+        _oracle_testvalue     = 'DROP TABLE "test"'
+        _postgres_testvalue   = 'DROP TABLE "test"'
+        _db2_testvalue        = 'DROP TABLE "test"'
+        _sqlserver_testvalue  = 'DROP TABLE test'
+        self.assertEqual(param.as_sql(DB_MYSQL), _mysql_testvalue, paramclass +'.as_sql(DB_MYSQL) failed.')
+        self.assertEqual(param.as_sql(DB_ORACLE), _oracle_testvalue, paramclass +'.as_sql(DB_ORACLE) failed.')
+        self.assertEqual(param.as_sql(DB_POSTGRESQL), _postgres_testvalue, paramclass +'.as_sql(DB_POSTGRESQL) failed.')
+        self.assertEqual(param.as_sql(DB_DB2), _db2_testvalue, paramclass +'.as_sql(DB_DB2) failed.')
+        self.assertEqual(param.as_sql(DB_SQLSERVER), _sqlserver_testvalue, paramclass +'.as_sql(DB_SQLSERVER) failed.')
+
+
 
 if __name__ == "__main__":
     #import sys;sys.argv = ['', 'Test.testName']
