@@ -8,7 +8,7 @@ Created on Oct 10, 2010
 import unittest
 from qal.dal.tests.framework import get_default_dal
 from qal.tools.meta_queries import Meta_Queries 
-from qal.dal.dal_types import DB_MYSQL, DB_POSTGRESQL, DB_ORACLE, DB_DB2, DB_SQLSERVER
+from qal.dal.types import DB_MYSQL, DB_POSTGRESQL, DB_ORACLE, DB_DB2, DB_SQLSERVER
 
 
 class TestSQL_Meta_queries(unittest.TestCase):
